/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Stefano Fornari
// @version $Id: Log.cpp,v 1.9 2006/01/16 16:19:13 toccy Exp $
//

#include "common/Log.h"

BOOL Log::debugEnabled = false;
BOOL Log::infoEnabled  = false;

Log LOG = Log(false);

wchar_t logmsg[512];

Log::Log(BOOL resetLog) {
    if (resetLog) {
        reset();
    }
    logPath[0] = 0;
}

Log::~Log() {
    if (logFile != NULL) {
        fclose(logFile);
    }
}

void Log::error(const wchar_t* msg) {
    if (infoEnabled == TRUE) {
        printMessage(LOG_ERROR, msg);
    }
}

void Log::info(const wchar_t* msg) {
    if (infoEnabled == TRUE) {
        printMessage(LOG_INFO, msg);
    }
}

void Log::debug(const wchar_t* msg) {
    if (debugEnabled == TRUE) {
        printMessage(LOG_DEBUG, msg);
    }
}

void Log::setDebug(BOOL enabled) {
    debugEnabled = enabled;
}

BOOL Log::isDebug() {
    return debugEnabled;
}

void Log::printMessage(const wchar_t* level, const wchar_t* msg) {

    wchar_t* currentTime = NULL;
    wchar_t  currentLogPath [256];

    currentTime = getCurrentTime(false);
    wsprintf(currentLogPath, TEXT("%s%s"), logPath, LOG_NAME);

    logFile     = _wfopen(currentLogPath, TEXT("a+"));
    fwprintf(logFile, TEXT("%s [%s] - %s\n"), currentTime, level, msg); fflush(logFile);
    fclose(logFile);

    delete[] currentTime;
}

void Log::reset() {

    wchar_t* currentTime = NULL;
    wchar_t  currentLogPath [256];

    currentTime = getCurrentTime(true);
    wsprintf(currentLogPath, TEXT("%s%s"), logPath, LOG_NAME);

    logFile     = _wfopen(currentLogPath, TEXT("w+"));
    fwprintf(logFile, TEXT("%s - # Funambol C++ Client API Native Log\n"), currentTime); fflush(logFile);
    fclose(logFile);

    delete[] currentTime;
}


void Log::setLogPath(wchar_t* configLogPath) {

    if (configLogPath != NULL) {
        wsprintf(logPath, TEXT("%s/"), configLogPath);
    } else {
        wsprintf(logPath, TEXT(""));
    }
}


/*
* return a the time to write into log file. If complete is true, it return
* the date too, else only hours, minutes, seconds and milliseconds
*/
wchar_t* Log::getCurrentTime(BOOL complete) {

    SYSTEMTIME sys_time;
    GetLocalTime(&sys_time);

    wchar_t fmtComplete[] = TEXT("%04d-%02d-%02d %02d:%02d:%02d:%03d");
    wchar_t fmt[]         = TEXT("%02d:%02d:%02d:%03d");

    wchar_t* ret = new wchar_t [64];

    if (complete) {
        wsprintf(ret, fmtComplete, sys_time.wYear, sys_time.wMonth, sys_time.wDay,
                      sys_time.wHour, sys_time.wMinute, sys_time.wSecond, sys_time.wMilliseconds);
    } else {
        wsprintf(ret, fmt, sys_time.wHour, sys_time.wMinute, sys_time.wSecond, sys_time.wMilliseconds);
    }
    return ret;

}