/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**

 How to test SSL connections on Win32
 -------------------------------------

 On the server:
 1) create the keystore:
    %JAVA_HOME%\bin\keytool -genkey -alias tomcat -keyalg RSA
 2) In $CATALINA_HOME/conf/server.xml uncomment the lines:
    <Connector className="org.apache.catalina.connector.http.HttpConnector"
               port="8443" minProcessors="5" maxProcessors="75"
               enableLookups="true"
               acceptCount="10" debug="0" scheme="https" secure="true">
      <Factory className="org.apache.catalina.net.SSLServerSocketFactory" clientAuth="false" protocol="TLS"/>
    </Connector>
 2) Export the certificate from the key store:
    %JAVA_HOME%\bin\keytool -export -alias tomcat -file myroot.cer  

 On the client:
 1) Connect (via https) to the server using a web-browser (type "https://<server_address>:8443)
 2) Accept and install the certificate sent from the server

**/

#ifndef INCL_TRANSPORT_AGENT
    #define INCL_TRANSPORT_AGENT

    #include "common/fscapi.h"

    #include "wininet.h"

    #include "http/common/URL.h"
    #include "http/common/Proxy.h"

    #define DEFAULT_MAX_TIMEOUT     300   //number of seconds of waiting response timeout.
    /*
     * This class is the transport agent responsible for messages exchange
     * over an HTTP connection.
     * This is a generic abtract class which is not bound to any paltform
     */

    class TransportAgent {

    private:
        URL url;
        Proxy proxy;

        unsigned int responseTimeout;

        int init();
        void uninit();
        wchar_t* postHTTP(URL& url, const wchar_t* msg);
        createHTTPRequest( char* request, const char*  resource, unsigned int length);

    public:

        TransportAgent(URL& url, Proxy& proxy, BOOL useCheckConnection, unsigned int maxResponseTimeout);
        ~TransportAgent();

        /*
         * Change the URL the subsequent calls to setMessage() should
         * use as target url.
         *
         * @param url the new target url
         */
        void setURL(URL& newURL);

        /*
         * Returns the url.
         */
        URL getURL();

        /*
         * Sends the given SyncML message to the server specified
         * by the instal property 'url'. Returns the response status code.
         * Use getResponse() to get the server response.
         */
        wchar_t* sendMessage(wchar_t* msg);

        void setResponseTimeout(unsigned int responseTimeout);

        unsigned int getResponseTimeout();
    };

    //
    // TBR
    //
    wchar_t* sendMessage(URL& url, const wchar_t* msg);

#endif